/*
 * --------------------------------------------
 * RBC::GUI::WebDialog:
 *   Wrapper to communicate with UI::WebDialog.
 *
 * Author : Sugar
 * Version: 1.2
 * Date   : 2017.10.1
 *
 * --------------------------------------------
 * WebDialogX:
 *   @see https://sketchucation.com/forums/viewtopic.php?f=180&t=46626&p=417591&hilit=WebDialogX#p417591
 *
 * --------------------------------------------
 *
 */

/*
 * **************************
 * ERROR
 * **************************
 * */

/*
 * Capture global JS errors and output to the console of SketchUp.
 * */
window.onerror = function (message, source, lineno, colno, error) {
    var data = {
        "message": message.toString(),
        "source": source.toString(),
        "lineno": lineno,
        "colno": colno.toString(),
        "error": error.toString()
    };
    RBC.GUI.WebDialog.callback('_error', data);
};

/*
 * **************************
 * console.log
 * **************************
 * */

/*
 *
 * */
function log_info(message) {
    if (window.console != undefined) {
        console.log(message);
    }
}

/*
 * **************************
 * Language
 * **************************
 * */


/*
 * */
var RBC = RBC || {};

//
RBC.Language = {

    /*
     * */
    strings: {},

    /* refresh language value.
     * @param [Function] callbackFun
     * @return [null]
     * */
    refresh: function (callbackFun) {
        log_info("refresh RBC.Language.");
        if (/SketchUp/i.test(navigator.userAgent)) {
            if (typeof(callbackFun) == "function") {
                RBC.Language._refreshFun = callbackFun;
            }
            RBC.GUI.WebDialog.callback({
                name: '_language',
                data: true,
                success: function (data) {
                    //typeof null返回"object"
                    if(typeof(data.strings) === 'object' && data.strings !== null){
                        RBC.Language._refreshFun(data.strings)
                    }
                },
                test:function (data) {

                }
            });
            RBC.GUI.WebDialog.callback('_language', true);
        } else {
            callbackFun.call(RBC.Language)
        }
        return null;
    },

    /* Get language value.
     * @method tr
     * @param [String] key
     * @return [String]
     * @since 2.0.0
     * */
    tr: function (key) {
        if (!/SketchUp/i.test(navigator.userAgent)) {
            log_info("RBC.Language: " + key + "<==>" + key);
            return key;
        }
        if (this.strings[key]) {
            log_info("RBC.Language: " + key + "<==>" + this.strings[key]);
            return this.strings[key];
        } else {
            RBC.GUI.WebDialog.callback('_language_insert_key', key);
            return key;
        }
    },

    /* Refresh callback function.
     * */
    _refreshFun: function () {
    }

};


/**
 * @name Settings
 * @typedef Settings
 * @class
 */
RBC.Settings = {
    properties: {},
    refreshFun:function () {

    },

    /**
     * Retrieves a value of a property by its name.
     * If the property does not exist, the property is created with the given value.
     * @param   {string}  key       The name of the property
     * @param   {object}  value      The new value to set
     * @returns {object}  The value
     */
    set: function (key, value) {
        if (properties[key]) {
            properties[key];
        } else {
            RBC.GUI.WebDialog.callback('_settings_insert_key', {
                key:key,
                value:value
            })
        }
        return value;
    },

    /**
     * Retrieves a value of a property by its name.
     * If the property does not exist, an empty property is created.
     * @param   {string}  key          The name of the property
     * @param   {object=} defaultValue  An optional default value to return if the property does not exist.
     * @returns {object}  The value
     */
    get: function (key, defaultValue) {
        if (properties[key]) {
            return properties[key];
        } else {
            return defaultValue;
        }
    },

    refresh:function (settings, callFun) {
        RBC.GUI.WebDialog.callback({
            name:'_settings',
            data:{
                'data':settings
            },
            success:function (data) {
                RBC.Settings.properties= data.properties;
                RBC.Settings.refreshFun(RBC.Settings);
            }
        })
    }
};


RBC.GUI = RBC.GUI || {};

/*
 * **************************
 * WebDialog
 * **************************
 * */

/*
 * @module RBC.GUI.WebDialog
 *
 * */
RBC.GUI.WebDialog = {

    version: 3,

    isSu: /SketchUp/i.test(navigator.userAgent),

    isSafari: navigator.userAgent.indexOf("Safari") > -1 && navigator.userAgent.indexOf("Chrome") == -1, //判断是否Safari浏览器

    /* @method close
     * @return {Boolean}
     * @version 2.2.0
     * */
    // close this dialog.
    close: function () {
        if (this.isSu) {
            this.callback('close', true);
            return true
        } else {
            log_info('The dialog can only be closed in SketchUp.');
            return false
        }

    },

    /* Wrapper to call any defined action_callback, handles escaping and encoding etc.
     * @params {Object} json
     * {
     *   name: string,
     *   data: json,
     *   test: function,
     *   success: function,
     *   error: function,
     *   complete: function
     * }
     * @params name, *arguments
     * @return {Boolean}
     * @since 2.2.0
     */
    callback: function () {
        // merge the given arguments with
        var opt = {
                data: null,
                success: function () {
                }, // null,
                error: function () {
                }, // null,
                complete: function () {
                }  // null
            },
            name = null,
            args = [];
        if (arguments.length == 1 && typeof(arguments[0]) === 'object') {
            json = arguments[0];
            for (var key in json) {
                opt[key] = json[key]
            }
            name = opt.name;
            args.push(opt.data);
            //
            if (!this.isSu) {
                if (typeof opt.test !== 'undefined') {
                    value = opt.test(args);
                    opt.complete(value);
                    return true;
                } else {
                    log_info('Error(J): Please add the Test function key!');
                    return false;
                }
            }
        } else if (arguments.length >= 1) {
            name = arguments[0];
            args = Array.prototype.slice.call(arguments).slice(1);
            //
            if (!this.isSu) {
                log_info('Error(J): Please execute it in SU!');
                return false
            }
        } else if (arguments.length == 1 && typeof(arguments[0]) === "string") {
            name = arguments[0];
            args = null;
        } else {
            log_info('Error(J): Incorrect parameter!');
            return false
        }
        var id = (this._messageIdCounter++);
        this._callbacks[id] = {
            success: opt.success,
            error: opt.error,
            complete: opt.complete
        };
        this._createMessageField(id, JSON.stringify(args));

        /*
         * OSX-Safari skips skp urls if they happen in a too short time interval.
         * We pass all skp urls through a queue that makes sure that a message is only
         * sent after the SketchUp side has received the previous message.
         * */
        this._messages.push([name, id]);
        if (this._ready) {
            this._nextMessage()
        }
        return true;
    },

    /* Call a Ruby method (even if a specific action_callback has not been defined)
     * Arguments: name and any amount of arguments  or  options object.
     *
     */
    callTo: function () {
        if (arguments.length == 1 && typeof(arguments[0]) === 'object') {
            var opt = arguments[0];
            opt.data = [opt.name, opt.data];
            opt.name = "_callTo";
            this.callback(opt);
        } else if (arguments.length >= 1) {
            var name_and_args = Array.prototype.slice.call(arguments).slice();
            name_and_args.unshift("_callTo");
            this.callback.apply(this, name_and_args);
        } else {
            return false
        }
    },


    /* Execute any Ruby code (even if a specific action_callback has not been defined)
     */
    execScript: function (code_string, options) {
        if (typeof(code_string) !== 'string') {
            return false
        }
        if (typeof(options) === 'object') {
            options.data = code_string;
            options.name = "_execScript";
            return this.callback(options);
        } else {
            return this.callback("_execScript", code_string);
        }
    },

    _nextMessage: function () {
        var message = this._messages.shift();
        if (!message) {
            this._ready = true;
            return;
        }
        this._ready = false;
        var url = "skp:" + message[0] + "@" + this._id_label + message[1];
        this._setLocation(url);
    },

    _ready: true,
    _messages: [],
    // Keeps track of callbacks.
    _messageIdCounter: 0,
    _callbacks: {},
    _id_label: '__<GUID>__',

    _createMessageField: function (id, value) {
        var messageField = document.createElement('input');
        messageField.setAttribute('type', 'hidden');
        messageField.setAttribute('style', 'display:none');
        messageField.setAttribute('id', 'RBC.GUI.WebDialog.' + id);
        messageField.value = value;
        document.body.appendChild(messageField);
    },
    _setLocation: function (url) {
        // allow the DOM to refresh so that the MessageField is created (from DynamicComponents dcBridge.js)
        window.setTimeout(function () {
            window.location.href = url;
        }, 0);
    },
    _cleanUp: function (id) {
        try {
            var elem = document.getElementById('RBC.GUI.WebDialog.' + id);
            elem.parentNode.removeChild(elem);
        } catch (e) {
            window.location = 'skp:_error@' + "Error(J): RBC.GUI.WebDialog._cleanUp:\\n" + e;
        } finally {
            delete this._callbacks[id];
        }
    },
    _cleanUpScripts: function () {
        // timeout to wait after injected script has been executed (otherwise this one is not removed)
        window.setTimeout(function () {
            var scripts = document.body.getElementsByTagName("script");
            for (var i = 0; i < scripts.length; i++) {
                scripts[i].parentNode.removeChild(scripts[i])
            }
        }, 0);
    },
    _fromRuby: function (id, functionName, argumentsString) {
        var returnValue = null;
        try {
            //Create the function
            var fn = eval(functionName); // alternative only for global functions: window[functionName]
            if (fn == null) {
                throw "function is not defined"
            }
            //argumentsString = argumentsString;
            var data = eval(argumentsString);
            returnValue = fn.apply(fn, data);
        } catch (e) {
            //alert("Error(J): RBC.GUI.WebDialog._fromRuby:\\n" + e);
            window.location = 'skp:_error@' + "Error(J): RBC.GUI.WebDialog._fromRuby:\\n" + e
        } finally {
            this._createMessageField(id, JSON.stringify(returnValue));
        }
    },
    _execScript: function (data) {
        var returnValue = null;
        try {
            returnValue = eval(data);
        } catch (e) {
            window.location = 'skp:_error@' + "Error(J): RBC.GUI.WebDialog._execScript:\\n" + e
        }
        return returnValue;
    }
};